#! /usr/bin/env python
#
# Bagisas
#
# A GUI for yafray (www.yafray.org)
#
# Copyright (C) 2003 Wolfram Kraus (schlops@web.de)
#
# Very special thanks to ray_theway for great inspiration and some code
#
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

import os, os.path, string, sys, time, threading
import ConfigParser
import Render
from FileDialog import *
from PrefsDialog import *
from xml.dom.ext.reader.Sax2 import FromXmlFile
from wxPython.wx import *
from wxPython.grid import *
from wxPython.lib.evtmgr import eventManager

VERSION = '0.0.2'

# IDs:
#  Menu
#  - File
ID_FNEW    = wxNewId()
ID_FOPEN   = wxNewId()
ID_FSAVE   = wxNewId()
ID_FSAAS   = wxNewId()
ID_FPREF   = wxNewId()
ID_FPRES   = wxNewId()
ID_FEXIT   = wxNewId()

#  - Tree
ID_TCOLA   = wxNewId()
ID_TCOLC   = wxNewId()
ID_TEXPA   = wxNewId()
ID_TEXPC   = wxNewId()

#  - Render
ID_RNDR    = wxNewId()
ID_RNTH    = wxNewId()
#  - Help
ID_HABOUT  = wxNewId()
#  Toolbar

MENU_STYLE = wxMENU_TEAROFF

SELFCLOSING = ('input', 'depth', 'color1', 'color2', 'filename', 'color',
			   'specular', 'reflection', 'hard', 'transmission', 'IOR',
			   'min_refle', 'reflected', 'transmited', 'transmitted',
			   'from', 'to', 'a', 'b', 'c', 'd', 'p', 'f', 'caus_tcolor',
			   'caus_rcolor', 'center', 'include', 'up', 'fog_color',
			   'outfile')

class Bagisas(wxFrame):
	''' The main class, responsible for the window layout and eventhandling'''
	
	def __init__(self, parent, id, title, file = None):
		''' Constructor '''
		self.dirname            = os.path.abspath(os.path.dirname(file))
		self.file               = os.path.basename(file)
		self.treeroot           = None
		self.rendering          = 0
		self.changed            = 0
		self.outfile = 'salida.tga'
		self.config = {	'render.yafex' : 'yafray',
						'render.numcpu': 1,
						'render.netop' : 0}

		self.loadConfig(['bagisas.ini', os.path.expanduser('~/.bagisasrc')])
		
		wxFrame.__init__(self,
						 parent,
						 id,
						 title,
						 style=wxDEFAULT_FRAME_STYLE|wxNO_FULL_REPAINT_ON_RESIZE)
		
		self.SetDimensions(-1, -1, 800, 600)
		self.SetSizeHints(320, 200)
		
		sb = self.CreateStatusBar() 
		
		# Setting up the menus
		filemenu= wxMenu()
		filemenu.Append(ID_FNEW   , '&New\tCtrl-X',      ' Create A New Yafray-File')
		filemenu.Append(ID_FOPEN  , '&Open\tF1',         ' Open An Existing Yafray-File')
		filemenu.AppendSeparator()
		filemenu.Append(ID_FSAVE  , '&Save',             ' Save The Current Yafray-File')
		filemenu.Append(ID_FSAAS  , 'Save &As...\tF2',   ' Save The Current Yafray-File As ...')
		filemenu.AppendSeparator()
		filemenu.Append(ID_FPREF  , '&Preferences',      ' Set The Render Preferences')
		filemenu.Append(ID_FPRES  , 'Save Pre&ferences', ' Save The Render Preferences For Future Sessions')
		filemenu.AppendSeparator()
		filemenu.Append(ID_FEXIT  , 'E&xit',             ' Terminate The Program')
		
		treemenu= wxMenu()
		treemenu.Append(ID_TCOLA  , '&Collapse Complete Tree',    ' Collapse every treenode')
		treemenu.Append(ID_TCOLC  , 'C&ollapse Current Node',     ' Collapse the current selected treenode')
		treemenu.AppendSeparator()
		treemenu.Append(ID_TEXPA  , '&Expand Complete Tree',      ' Expand every treenode')
		treemenu.Append(ID_TEXPC  , 'E&xpand Current Node',       ' Expand the current selected treenode')

		rndrmenu= wxMenu()
		rndrmenu.Append(ID_RNDR   , 'Render\tF12',         ' Render')
		# rndrmenu.Append(ID_RNTH   , 'Thread',         ' Thread')
		
		helpmenu= wxMenu()
		helpmenu.Append(ID_HABOUT , '&About',      ' Informations About This Nice Little Program')
		
		# Create the menubar
		menuBar = wxMenuBar()
		menuBar.Append(filemenu,'&File') 
		menuBar.Append(treemenu,'&Tree') 
		menuBar.Append(rndrmenu,'&Render') 
		menuBar.Append(helpmenu,'&Help') 
		self.SetMenuBar(menuBar)  
		
		#
		# Split the main window
		splitMAIN   = wxSplitterWindow(self, -1, style = wxSP_LIVE_UPDATE)
		splitMAOP   = wxSplitterWindow(splitMAIN, -1, style = wxSP_3DSASH|wxSP_LIVE_UPDATE)
		splitATCO   = wxSplitterWindow(splitMAOP, -1, style = wxSP_3DSASH|wxSP_LIVE_UPDATE)
		
		# The DOM Tree
		tID = wxNewId()
		self.tree = DOMTreeCtrl(self, splitMAOP, tID, wxDefaultPosition, wxDefaultSize,
								wxTR_HAS_BUTTONS
								#| wxTR_MULTIPLE
								#| wxTR_MULTIPLE
								#| wxTR_HIDE_ROOT
								)
		
		self.createDOMTree()
		# self.tree.SetPyData(self.treeroot, None)
		
		if self.treeroot:
			self.tree.Expand(self.treeroot)
		
		# The attribute-list
		tID = wxNewId()
		self.attrgrid = AttribGrid(self, splitATCO, tID,
									   style=wxLC_REPORT)#|wxLC_VRULES|wxLC_HRULES)
		
		# The preview

		self.preview = wxStaticBitmap(splitATCO, -1,
									  wxImage('default.png', wxBITMAP_TYPE_PNG).ConvertToBitmap(),
									  wxDefaultPosition, wxDefaultSize, style=wxSUNKEN_BORDER)

		# The output
		self.output = wxTextCtrl(splitMAIN, -1, style = wxTE_READONLY|wxTE_MULTILINE)
		
		splitATCO.SetMinimumPaneSize(20)
		splitATCO.SplitHorizontally(self.attrgrid, self.preview, self.GetClientSize()[1]/2)

		splitMAOP.SetMinimumPaneSize(20)
		splitMAOP.SplitVertically(self.tree, splitATCO, self.GetClientSize()[0]/2)
		
		splitMAIN.SetMinimumPaneSize(20)
		splitMAIN.SplitHorizontally(splitMAOP, self.output, 5*self.GetClientSize()[1]/6)
		
		# Set the Renderer
		self.render = None
		
		# Do the eventhandling
		self.setEventHandling()
		
		# Finally...
		self.CentreOnScreen()
		self.Show(1)
		
		
	def setEventHandling(self):
		''' Connects GUI objects with events'''
		# Eventhandling
		eventManager.Register(self.onClose,    EVT_CLOSE, self)
		# - Menus
		eventManager.Register(self.onNew,      EVT_MENU, win=self, id=ID_FNEW)
		eventManager.Register(self.onOpen,     EVT_MENU, win=self, id=ID_FOPEN)
		eventManager.Register(self.onSave,     EVT_MENU, win=self, id=ID_FSAVE)
		eventManager.Register(self.onSaveAs,   EVT_MENU, win=self, id=ID_FSAAS)
		eventManager.Register(self.onPrefs,    EVT_MENU, win=self, id=ID_FPREF)
		eventManager.Register(self.onSavePrefs,EVT_MENU, win=self, id=ID_FPRES)
		eventManager.Register(self.onExit,     EVT_MENU, win=self, id=ID_FEXIT)
		eventManager.Register(self.onAbout,    EVT_MENU, win=self, id=ID_HABOUT)
		eventManager.Register(self.onRender,   EVT_MENU, win=self, id=ID_RNDR)
		eventManager.Register(self.onThread,   EVT_MENU, win=self, id=ID_RNTH)
		eventManager.Register(self.onCollapse, EVT_MENU, win=self, id=ID_TCOLA)
		eventManager.Register(self.onCollaC,   EVT_MENU, win=self, id=ID_TCOLC)
		eventManager.Register(self.onExpand,   EVT_MENU, win=self, id=ID_TEXPA)
		eventManager.Register(self.onExpandC,  EVT_MENU, win=self, id=ID_TEXPC)
		
	def loadConfig(self, file):
		''' Opens An Existing Configfile If Available And Sets The Values'''
		cp = ConfigParser.ConfigParser(self.config)
		cp.read(file)
		for sec in cp.sections():
			name=string.lower(sec)
			for opt in cp.options(sec):
				self.config[name+'.'+string.lower(opt)]=string.strip(cp.get(sec, opt))
		
	def updateViews(self):
		''' Updates The Value/Attribute Grid If Another Treenode Has Been Selected'''
		data = self.tree.GetPyData(self.tree.GetSelection())
		if data:
			# self.dataView.SetValue(data.get('text', ''))
			self.attrgrid.ClearGrid()
			self.attrgrid.BeginBatch()
			al = data.get('attributes', {})
			keys = al.keys()
			keys.sort()
			idx = 0
			for x in keys:
				self.attrgrid.SetCellValue(idx, 0, x)
				self.attrgrid.SetCellValue(idx, 1, al[x])
				idx += 1
			for x in range(idx, self.attrgrid.GetColSize(0)):
				self.attrgrid.SetCellValue(x, 0, '')
				self.attrgrid.SetCellValue(x, 1, '')
			
			self.attrgrid.EndBatch()
			self.attrgrid.AdjustScrollbars()
		else:
			self.attrgrid.Clear()
			#self.dataView.SetValue('')
			
	def onNew(self, e):
		if self.changed:
			dlg = wxMessageDialog(self,
								  'There are unsaved changes. Do you really want to create a new file?',
								  'Create new file?',
								  wxYES_NO | wxNO_DEFAULT | wxICON_QUESTION) 
			if dlg.ShowModal() == wxID_YES: 
				self.createNew()
		else:
			self.createNew()

	def createNew(self):
		self.tree.DeleteChildren(self.treeroot)
		self.file=''
		self.SetTitle('Bagisas v'+VERSION)
		self.setChanged(0)
			
	def onSave(self, e):
		if self.file != '':
			self.save(os.path.join(self.dirname, self.file))
		else:
			self.onSaveAs(e)
			
	def onSaveAs(self, e):
		wildcard = 'XML Files(*.xml)|*.xml|All files (*.*)|*.*'
		dlg = FileDialog(self, 'Save As...', self.dirname, self.file, wildcard, wxSAVE|wxOVERWRITE_PROMPT)
		if dlg.ShowModal() == wxID_OK:
			self.save(dlg.GetPath())
			self.dirname = os.path.dirname(dlg.GetPath())
			self.file    = os.path.basename(dlg.GetPath())
	
	def save(self, fn, reset = 1):
		file = open(fn, 'w')
		root = self.tree.GetRootItem()
		print >> file,'<'+self.tree.GetItemText(root)+'>'
		if self.tree.ItemHasChildren(root):
			self.traverseTree(file, root, 1, '  ')
		print >> file,'</'+self.docroot.tagName+'>'
		if reset:
			self.setChanged(0)
		file.close()
		
	def  traverseTree(self, file, root, cookie, spacing):
		if self.tree.ItemHasChildren(root):
			child, nco = self.tree.GetFirstChild(root, cookie)
			for n in xrange(self.tree.GetChildrenCount(root, 0)):
				tr = self.tree.GetPyData(child)
				print >> file,spacing+'<'+tr['name'],
				al = tr.get('attributes', {})
				for x in al.keys():
					print >> file, x+'="'+al[x]+'"',
				if tr['name'] in SELFCLOSING:
					print >> file,'/>'
				else:
					print >> file,'>'
					self.traverseTree(file, child, nco, spacing+' ')
					print >> file, spacing+'</'+tr['name']+'>'
				child = self.tree.GetNextSibling(child)
		
	def updateShaders(self, root, cookie, oldval, newval):
		if self.tree.ItemHasChildren(root):
			child , nco = self.tree.GetFirstChild(root, cookie)
			for n in xrange ( self.tree.GetChildrenCount ( root, 0 ) ):
				data = self.tree.GetPyData(child)
				if data['name'] == u'object':
					sn = data['attributes'].get(u'shader_name', 0)
					if sn == oldval:
						data[u'attributes'][u'shader_name'] =  newval
						self.tree.SetPyData(child, data)
				else:
					self.updateShaders(child, nco, oldval, newval)
				child = self.tree.GetNextSibling(child)

	def onOpen(self, e):
		''' Open A File'''
		wildcard = 'XML Files(*.xml)|*.xml|All files (*.*)|*.*'
		dlg = FileDialog(self, 'Choose a file', self.dirname, '', wildcard, wxOPEN|wxFILE_MUST_EXIST)
		if dlg.ShowModal() == wxID_OK:
			self.file=os.path.basename(dlg.GetPath())
			self.dirname=os.path.dirname(dlg.GetPath())
			self.attrgrid.ClearGrid()
			self.tree.DeleteAllItems()
			self.SetTitle('Bagisas v'+VERSION+' '+self.file)
			self.createDOMTree()
			self.tree.Expand(self.tree.GetRootItem())
		dlg.Destroy()
		
	def onClose(self, e):
		''' Ends The Application'''
		if self.changed:
			dlg = wxMessageDialog(self, 'There are unsaved changes. Do you really want to end?', 'Exit Program?',
								  wxYES_NO | wxNO_DEFAULT | wxICON_QUESTION) 
			if dlg.ShowModal() == wxID_YES: 
				print 'Byebye!'
				while not self.Destroy():
					print 'Please wait.'
			else:
				dlg.Destroy()
		else:
			print 'Byebye!'
			while not self.Destroy():
				print 'Please wait.'
			
	def onExit(self, e):
		self.Close(1) 
		
	def onPrefs(self, evt):
		dlg = PrefsDialog(self, self, -1, "Preferences", size=wxSize(500, 300),
					   style = wxCAPTION | wxSYSTEM_MENU)
		val = dlg.ShowModal()
		if val == wxID_OK:
			self.config['render.yafex']  = dlg.getYafEx()
			self.config['render.numcpu'] = dlg.getNumCpu()
			self.config['render.netop']  = dlg.getNetOp()

	def onSavePrefs(self, evt):
		if os.name == 'nt' or os.name == 'dos':
			fo = open('bagisas.ini', 'w')
		else:
			fo = open(os.path.expanduser('~/.bagisasrc'), 'w')
		cp = ConfigParser.ConfigParser(self.config)
		cp.write(fo)
		
	def onAbout(self, e):
		''' Shows About-Dialog'''
		d= wxMessageDialog( self, ' A YAFRAY browser \n'
							' in wxPython','About Bagisas', wxOK)
		d.ShowModal() 
		d.Destroy() 
		
	def onRender(self, e):
		''' Renders The Current File'''
		try:
			if not self.changed:
				self.render = Render.Render(self, os.path.join(self.dirname, self.file), self.outfile)
				self.render.start()
			else:
				add = time.time()
				tempfn = os.path.join(self.dirname,'tmp%f.xml' % (add))
				self.save(tempfn, 0)
				self.render = Render.Render(self, tempfn, self.outfile, 1)
				self.render.start()
		except:
			dlg = wxMessageDialog(self.parent,
								  'An error has occured during rendering!',
								  'ERROR!',
								  wxOK | wxICON_ERROR)
			ok = dlg.ShowModal()

			
	def onThread(self, e):
		print '------------------------------------------------------------'
		print threading.activeCount()
		print self.rendering
		print self.render.getName()
		print self.render.isAlive()
		
	def onCollapse(self, evt):
		self.tree.collapseNode(self.tree.GetRootItem())

	def onCollaC(self, evt):
		node = self.tree.GetSelection()
		if node.IsOk():
			self.tree.collapseNode(node)

	def onExpand(self, evt):
		self.tree.expandNode(self.tree.GetRootItem())
		
	def onExpandC(self, evt):
		node = self.tree.GetSelection()
		if node.IsOk():
			self.tree.expandNode(node)

	def createDOMTree(self):
		if self.file:
			self.doc  = FromXmlFile(os.path.join(self.dirname, self.file))
			self.docroot = self.doc.documentElement
			self.treeroot = self.tree.AddRoot(self.docroot.tagName)
			data = {'attributes': self.docroot.attributes}
			self.tree.SetPyData(self.treeroot, data)
			for c in self.docroot.childNodes:
				self.addChild(self.treeroot, c)
			self.tree.SortChildren(self.treeroot)
			
	def addChild(self, parent, child):
		if child.nodeType == child.ELEMENT_NODE :
			data = {'name': child.tagName}
			data['attributes'] = self.attribToDict(child.attributes)
			if child.tagName == 'render':
				self.outfile =  data['attributes'].get('outfile', 'salida.tga')
			found = 0
			x = 0
			add = data['attributes'].get('name', '')
			if add != '':
				add = ' ' + add
			ct = self.tree.AppendItem(parent, child.tagName+add)
			self.tree.SetPyData(ct, data)
			if child.hasChildNodes():
				for c in child.childNodes:
					self.addChild(ct, c)
		elif child.nodeType == child.TEXT_NODE:
			if string.strip(child.data) != '':
				data = self.tree.GetPyData(parent)
				data['text'] = string.strip(child.data)
				self.tree.SetPyData(parent, data)
			
				
	def attribToDict(self, attrib):
		rv = {}
		x = 0
		while x < attrib.length:
			rv[attrib.item(x).name] = attrib.item(x).value
			x += 1
		return rv
			
	def setChanged(self, change):
		if change and not self.changed:
			self.SetTitle('Bagisas v'+VERSION+' '+self.file+' [*]')
		if not change and self.changed:
			self.SetTitle('Bagisas v'+VERSION+' '+self.file)
		self.changed = change

class DOMTreeCtrl(wxTreeCtrl):
	
	def __init__(self, master, *args, **kwargs):
		apply(wxTreeCtrl.__init__,(self,) + args, kwargs)
		self.last = None
		self.changing = 0
		self.master = master
		eventManager.Register(self.onTreeSelChanged,  EVT_TREE_SEL_CHANGED,    self)
		eventManager.Register(self.onTreeSelChanging, EVT_TREE_SEL_CHANGING,   self)
		eventManager.Register(self.onTreeActivate,    EVT_TREE_ITEM_ACTIVATED, self)
		eventManager.Register(self.onKeyDown,         EVT_TREE_KEY_DOWN,       self)

	def onTreeSelChanged(self, evt):
		self.last = evt.GetOldItem()
		self.changing = 1
		self.master.updateViews()
		self.changing = 0
		
	def onTreeSelChanging(self, evt):
		self.last = evt.GetOldItem()
		
	def onTreeActivate(self, e):
		self.master.updateViews()

	def onKeyDown(self, e):
		# For some reasons EVT_TREE_DELETE_ITEM isn't activated until the end of the application
		if e.GetKeyCode() == WXK_DELETE:
			if self.GetSelection() != self.GetRootItem():
				self.Delete(self.GetSelection())
			else:
				self.DeleteChildren(self.GetSelection())
			self.master.setChanged(1)
		else:
			e.Skip()
			
	def collapseNode(self, node):
		self.treeWork(self.Collapse, node, 0)
		self.SelectItem(node)
		
	def expandNode(self, node):
		self.treeWork(self.Expand, node, 0)

	def treeWork(self, func, inital, cookie):
		if inital != self.GetRootItem():
			func(inital)
		getter = self.GetFirstChild
		nc = self.GetChildrenCount ( inital, 0 ) 
		for i in xrange ( nc ):
			child, cookie = getter(inital, cookie)
			getter = self.GetNextChild
			func(child)
			if self.GetChildrenCount(child) > 0:
				self.treeWork(func, child, cookie)
				
	def OnCompareItems(self, item1, item2):
		compdict = {'shader': 0, 'texture': 1,    'modulator': 2, 'light': 3,
					'filter': 4, 'transform': 5,  'object': 6,    'include': 7,
					'camera': 8, 'background': 8, 'render': 10}
		if compdict[self.GetPyData(item1)['name']] ==  compdict[self.GetPyData(item2)['name']]:
			if self.GetItemText(item1) < self.GetItemText(item2): return -1
			if self.GetItemText(item1) == self.GetItemText(item2): return 0
			return 1 
		if compdict[self.GetPyData(item1)['name']] <  compdict[self.GetPyData(item2)['name']]:
			return -1
		return 1
	
class AttribGrid(wxGrid):
	
	def __init__(self, master, *args, **kwargs):
		apply(wxGrid.__init__,(self,) + args, kwargs)
		self.master = master

		table = AttribDataTable()
		self.SetTable(table, 1)
		
		self.SetRowLabelSize(0)
		self.AutoSizeColumns()
		self.SetMargins(50,50)
		self.Fit()
		attr0 = wxGridCellAttr()
		attr0.SetReadOnly(1)
		self.SetColAttr(0, attr0)
		attr1 = wxGridCellAttr()
		attr1.SetEditor(AttribGridEditor(self.master))
		self.SetColAttr(1, attr1)
		
		self.attribute = self.value = ''
		
		eventManager.Register(self.onLeftDClick,     EVT_GRID_CELL_LEFT_DCLICK, self)

	def onLeftDClick(self, evt):
		if self.CanEnableCellControl():
			self.EnableCellEditControl()
			

	def ClearGrid(self):
		self.GetTable().data = [['','']]
		self.DeleteRows(0, self.GetNumberRows()-1)
		
class AttribGridEditor(wxPyGridCellEditor):

	def __init__(self, master, *args, **kw):
		apply(wxPyGridCellEditor.__init__,(self,) + args, kw)
		self.master = master
		
	def Create(self, parent, id, evtHandler):
		"""
		Called to create the control, which must derive from wxControl.
		*Must Override*
		"""
		self._tc = wxTextCtrl(parent, id, "")
		self._tc.SetInsertionPoint(0)
		self.SetControl(self._tc)
		if evtHandler:
			self._tc.PushEventHandler(evtHandler)

	def BeginEdit(self, row, col, grid):
		"""
		Fetch the value from the table and prepare the edit control
		to begin editing.  Set the focus to the edit control.
		*Must Override*
		"""
		self.startValue  = grid.GetTable().GetValue(row, col)
		self.startAttrib = grid.GetTable().GetValue(row, col-1)
		self._tc.SetValue(self.startValue)
		self._tc.SetInsertionPointEnd()
		self._tc.SetFocus()

		self._tc.SetSelection(0, self._tc.GetLastPosition())

	def EndEdit(self, r, c, grid):
		changed = false

		val = string.strip(self._tc.GetValue())
		if val != self.startValue and val != '':
			changed = 1
			grid.GetTable().SetValue(r, c, val) # update the table

			if self.master.tree.changing:
				node = self.master.tree.last
			else:
				node = self.master.tree.GetSelection()
			data = self.master.tree.GetPyData(node)
			data['attributes'][self.startAttrib] = val
			self.master.tree.SetPyData(node, data)
			if self.startAttrib == u'name':
				# Update the tree-nodes
				add = ' ' + val
				self.master.tree.SetItemText(node, data['name']+add)
				self.master.tree.SortChildren(self.master.tree.GetRootItem())
			if self.startAttrib == u'outfile':
				self.master.outfile = val
			if data['name'] == u'shader' and self.startAttrib == u'name':
				self.master.updateShaders(self.master.tree.GetRootItem(), 0, self.startValue, val)

		self.startValue  = ''
		self.startAttrib = ''
		self._tc.SetValue('')
		self.master.setChanged(1)
		return changed

	def Reset(self):
		"""
		Reset the value in the control back to its starting value.
		*Must Override*
		"""
		self._tc.SetValue(self.startValue)
		self._tc.SetInsertionPointEnd()

	def Clone(self):
		"""
		Create a new object which is the copy of this one
		*Must Override*
		"""
		return AttribGridEditor(self.master.tree)
		

class AttribDataTable(wxPyGridTableBase):
	
	def __init__(self):
		wxPyGridTableBase.__init__(self)
		self.colLabels = ['Attribut', 'Value']
		self.dataTypes = [wxGRID_VALUE_STRING, wxGRID_VALUE_STRING]
		self.data      = [['', '']]*30
		
	# required methods for the wxPyGridTableBase interface
		
	def GetNumberRows(self):
		return len(self.data)
	
	def GetNumberCols(self):
		return len(self.data[0])
	
	def IsEmptyCell(self, row, col):
		try:
			return not self.data[row][col]
		except IndexError:
			return 1
		
	def GetValue(self, row, col):
		try:
			return self.data[row][col]
		except IndexError:
			return ''
		
	def SetValue(self, row, col, value):
		try:
			self.data[row][col] = value
		except IndexError:
			# add a new row
			self.data.append([''] * self.GetNumberCols())
			self.SetValue(row, col, value)
			
			if row >= self.GetView().GetNumberRows():
				# tell the grid we've added a row
				msg = wxGridTableMessage(self, wxGRIDTABLE_NOTIFY_ROWS_APPENDED, 1)
				self.GetView().ProcessTableMessage(msg)
			
	def DeleteRows(self, pos, num_rows):
		pass
	
	def Clear(self):
		pass
	
	# Some optional methods
	
	# Called when the grid needs to display labels
	def GetColLabelValue(self, col):
		return self.colLabels[col]
	
	# Called to determine the kind of editor/renderer to use by
	# default, doesn't necessarily have to be the same type used
	# nativly by the editor/renderer if they know how to convert.
	def GetTypeName(self, row, col):
		return self.dataTypes[col]
	
	# Called to determine how the data can be fetched and stored by the
	# editor and renderer.  This allows you to enforce some type-safety
	# in the grid.
	def CanGetValueAs(self, row, col, typeName):
		colType = self.dataTypes[col].split(':')[0]
		if typeName == colType:
			return 1
		else:
			return false
		
	def CanSetValueAs(self, row, col, typeName):
		return self.CanGetValueAs(row, col, typeName)
	
		
def main(file=''):
	app = wxPySimpleApp()
	frame = Bagisas(None, -1, 'Bagisas v'+VERSION+' '+file, file)
	app.MainLoop()
	
if __name__ == '__main__':
	if len(sys.argv) > 1:
		main(sys.argv[1])
	else:
		main()


## History:
##
##  v 0.0.1 13.Mar.2003 ('King Kong Five'):
##    - Inital Release
##
##  v 0.0.2 17.Mar.2003 ('See Dem Coming'):
##    - Tree/Node Expand/Collapse
##    - View rendered image
##    - Preferences Dialog
##    - shader name changed => automagically change object-attribute shader_name
##    - FIXED: Attribute Changes Weren't Recognized Until Save
##    - FIXED: outfile change resulted in not viewing the image
##
## TODO:
##  (In no particular order)
##
##  - Comments!!
##  - Color Dialogs for Color Values
##  - Improved Error Messages
##  - Add items
##  - Kill render-thread
##  - Improved output of render-thread
##  - Undo (??)
##  - Show all attributes of an element, even those optional which are not in the file
##  - Numpad+ in FileDialog
##  - MDI w/ Cut/Copy & Paste
##  - Metarial Editor w/ Preview of the material (??)
##  - Throbberwindow w/ Cancel while reading XML-File
##  - Text editor
