import os.path
from wxPython.wx import wxFrame, wxBoxSizer, wxDialog, wxSizer, wxStaticText
from wxPython.wx import wxTextCtrl, wxStaticLine, wxButton, wxNewId, wxPoint
from wxPython.wx import wxSlider, wxCheckBox, EVT_BUTTON, wxFileDialog
from wxPython.wxc import *
from wxPython.lib.evtmgr import eventManager

ID_BTN_YAFEX = wxNewId()

class PrefsDialog(wxDialog):
	
	def __init__(self, parent, *args, **kwargs):
		apply(wxDialog.__init__,(self,) + args, kwargs)
		self.parent = parent
		sizer = wxBoxSizer(wxVERTICAL)
		label = wxStaticText(self, -1, "Yafray Options")
		sizer.Add(label, 0, wxALIGN_LEFT|wxALL, 5)

		box = wxBoxSizer(wxHORIZONTAL)
		label = wxStaticText(self, -1, "Yafray Executable:")
		box.Add(label, 0, wxALIGN_RIGHT|wxALL, 3)
		self.yafex = wxTextCtrl(self, -1, self.parent.config['render.yafex'], size=(120,-1))
		box.Add(self.yafex, 1, wxALIGN_LEFT|wxALIGN_CENTER_VERTICAL|wxALL, 3)
		btn = wxButton(self, ID_BTN_YAFEX , "Browse...", wxPoint(0,0), style = wxBU_EXACTFIT)
		box.Add(btn, 1, wxALIGN_RIGHT|wxALL, 3)
		sizer.AddSizer(box, 0, wxGROW|wxALIGN_CENTER_VERTICAL|wxALL, 3)

		box = wxBoxSizer(wxHORIZONTAL)
		label = wxStaticText(self, -1, "Number Of CPUs To Use:")
		box.Add(label, 0, wxALIGN_RIGHT|wxALIGN_CENTER_VERTICAL|wxALL, 3)
		self.slid = wxSlider(self, -1, int(self.parent.config['render.numcpu']),
							 1, 16, style = wxSL_HORIZONTAL|wxSL_AUTOTICKS|wxSL_LABELS)
		box.Add(self.slid, 1, wxALIGN_CENTRE|wxALL, 3)
		sizer.AddSizer(box, 0, wxGROW|wxALIGN_CENTER_VERTICAL|wxALL, 3)

		box = wxBoxSizer(wxHORIZONTAL)
		label = wxStaticText(self, -1, "Use Net Optimized:")
		box.Add(label, 0, wxALIGN_RIGHT|wxALIGN_CENTER_VERTICAL|wxALL, 3)
		self.cb = wxCheckBox(self, -1, '')
		self.cb.SetValue(int(self.parent.config['render.netop']))
		box.Add(self.cb, 1, wxALIGN_CENTRE|wxALL, 3)
		sizer.AddSizer(box, 0, wxGROW|wxALIGN_CENTER_VERTICAL|wxALL, 3)

		line = wxStaticLine(self, -1, size=(20,-1), style=wxLI_HORIZONTAL)
		sizer.Add(line, 0, wxGROW|wxALIGN_CENTER_VERTICAL|wxRIGHT|wxTOP, 3)

		box = wxBoxSizer(wxHORIZONTAL)
		btn = wxButton(self, wxID_OK, " OK ")
		btn.SetDefault()
		box.Add(btn, 0, wxALIGN_CENTRE|wxALL, 5)
		btn = wxButton(self, wxID_CANCEL, " Cancel ")
		box.Add(btn, 0, wxALIGN_CENTRE|wxALL, 5)
		sizer.AddSizer(box, 0, wxALIGN_CENTER_VERTICAL|wxALIGN_CENTER_HORIZONTAL|wxALL, 5)

		self.SetSizer(sizer)
		self.SetAutoLayout(1)
		sizer.Fit(self)

		eventManager.Register(self.onBrowse, EVT_BUTTON, win=self, id=ID_BTN_YAFEX)

	def onBrowse(self, evt):
		dlg = wxFileDialog(self, "Choose Yafray binary", ".", "",
						   "Yafray Binary (yafray.exe)|yafray.exe|All Files (*.*)|*.*", wxOPEN)
		try:
			if dlg.ShowModal() == wxID_OK:
				bin = os.path.join(dlg.GetDirectory(),dlg.GetFilename())
				self.yafex.SetValue(bin)
		finally:
			dlg.Destroy()

	def getYafEx(self):
		return self.yafex.GetValue()

	def getNumCpu(self):
		return self.slid.GetValue()

	def getNetOp(self):
		return self.cb.GetValue()
